/*
 * libbsr - allocation services for the Barrier Synchronization Register
 *          on IBM POWER systems
 *
 * Copyright (C) 2008 Sonny Rao, IBM Corporation.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <dirent.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include "bsr.h"

static int print_interval = 1;

int bsr_test(int loop_count, int map_count)
{
	unsigned i, total, smask, free,fmask, bsr_stride = 0;
	unsigned id, bsr_length, alloc_size = 0;
	void *map;
	int count, ret;

	for (count = 0; count < loop_count; count++) {
		if (count % print_interval == 0)
			printf("\tAttempting bsr test %d\n", count);

		ret = bsr_query(&total, &smask, &free , &fmask );
		if (count % print_interval == 0) {
			printf("\ttotal = %d, smask = 0x%x, tfree = %d, "
			       "fmask = 0x%x\n", total, smask, free, fmask);
		}

		if (!fmask) {
			printf("\tno free BSRs\n");
			return 0;
		}

		/* find the smallest BSR and allocate it */
		for (i = 0; i < (sizeof(fmask) * CHAR_BIT); i++) {
			unsigned cur_size = 1 << i;
			if (cur_size & fmask) {
				alloc_size = cur_size;
				break;
			}
		}

		if (count % print_interval == 0)
			printf("\ttrying to alloc a %d byte BSR\n", alloc_size);
		ret = bsr_alloc(alloc_size, &bsr_stride, &id);
		if (!ret) {
			if (count % print_interval == 0)
				printf("\tstride = %d id = 0x%x\n",
				       bsr_stride, id);

			for (i = 0; i < map_count; i++) {
				map = bsr_map(NULL, id, 0, 0, &bsr_length);
				if (count % print_interval == 0)
					printf("\tmapping %d = %p\n\n", i, map);
				bsr_unmap(map, bsr_length);
			}

			bsr_free(id);
		} else {
			printf("\tret = %d, %s\n", ret, strerror(ret));
			return ret;
		}
	}

	return 0;
}

int main()
{
	int ret;

	printf("Testing simple bsr alloc/map\n");
	ret = bsr_test(1, 1);
	if (ret)
		return ret;

	printf("Testing multiple mappings\n");
	ret = bsr_test(1, 10);
	if (ret)
		return ret;

	printf("Testing bsr bomb\n");
	print_interval = 1000;
	ret = bsr_test(100000, 1);
	if (ret)
		return ret;

	return ret;
}
